@@ -4,7 +4,7 @@ use strict; use warnings;
 use base 'Exporter';
 use File::Find;
 
-our @EXPORT = qw( purge_dir strip_copyright );
+our @EXPORT = qw( purge_dir strip_copyright strip_shebang );
 
 sub purge_dir {
 }
@@ -31,3 +31,9 @@ sub strip_copyright {
     return $line;
 }
 
+sub strip_shebang {
+    my $line = shift;
+    $line    =~ s/^#!.*//;
+    return $line;
+}
+
